/********************************************************************************************
(C) Jakub Borzdynski 2006
jotbeage@interia.pl
-----------------------------
Szkola Konstruktorow 137 - g-meter
********************************************************************************************/
#define sbi(port, bit) port|=(1<<bit)
#define cbi(port, bit) port&=~(1<<bit)
#define F_CPU 1000000UL

#include <avr/io.h> //definicje rejestrow uK
#include <avr/signal.h>
#include <util/delay.h>
#include "src/lcd.h"
#include "src/adxl.h"
#include "src/eeprom.h"

//obiekt
adxl pomiar ;
unsigned char maska = 0x00 ;

SIGNAL(SIG_OVERFLOW0){
	wysw.refresh() ;
	PORTD ^= maska ;
}

//-------------------------------------------------------------------------------------------------------------
int main(){
	//port wejsciowy baterii
	cbi(DDRD, 3) ;
	sbi(PORTD,3) ;
	
	//podtrzymanie przetwornicy
	sbi(DDRD, 4) ;
	sbi(PORTD,4) ;
	
	//porty buzzera
	DDRD |= 0x03 ;
	PORTD &= 0xFC ;
	PORTD |= 0x01 ;
	
	//konfiguracja klawiatury
	DDRC  &= 0xFC ;
	PORTC |= 0x03 ;
	
	//pomiar
	unsigned int wynik = 0 ;

	//zlicza czas przytrzymania przycisku
	unsigned int konfig = 0 ;
	unsigned int zmien_prog = 0 ;
	unsigned char program = pamiec[EEPROM_ADR_PROGRAM] ; //okresla, jaki realizowac program
	unsigned int dzwiek = 0 ; //zmienna wymuszajaca pulsowanie dzwieku
	unsigned int przetwornica = 0 ; //okresla kiedy wylaczyc przetwornice
	
	//petla glowna
	while(1){
		//resetowanie watchdog'a
		asm("wdr") ;
		
		//realizuj zalozony program
		switch(program%3){
			case POMIAR_G:
				wynik = pomiar(POMIAR_G) ;
			break ;
			
			case POMIAR_G_MAX:
				wynik = pomiar(POMIAR_G_MAX) ;
			break ;
			
			case POMIAR_ST:
				wynik = pomiar(POMIAR_ST) ;
			break ;
		}
		
		//zapisz wynik
		wysw(wynik) ;
		
		//sprawdzanie, czy wejsc do trybu konfiguracji
		if(bit_is_clear(PINC, 1)){konfig++;}else{konfig=0;}
		if(konfig>1000){
			wysw(LCD_CFN) ;
			for(unsigned char i=0 ; i<100 ; i++){
				asm("wdr") ;
				_delay_ms(10) ;
			}
			pomiar.konfiguracja() ;
		}
		
		//zmiana programu ?
		if(bit_is_clear(PINC, 0)){zmien_prog++;}else{zmien_prog=0;}
		if(zmien_prog>50){
			program++ ;
			
			//wyswietl info, ze zmieniono program
			wysw(LCD_PRC) ;
			for(unsigned char i=0 ; i<100 ; i++){
				asm("wdr") ;
				_delay_ms(10) ;
			}
			
			//zapamietaj ten program
			pamiec.zapisz(EEPROM_ADR_PROGRAM, program) ;
		}
		
		//sygnalizacja poziomu (plus minus 1 stopien), dzwiek pulsujacy
		if( ((wynik<=1) || (wynik==359)) && (++dzwiek&0x0080)){
			//ustawienie portow
			if((PORTD&0x03)==0x00){
				PORTD |= 0x01 ;
			}
			
			//maska
			maska = 0x03 ;
			przetwornica = 0 ; //blokuj mozliwosc wylaczenia
		}else{
			//wylaczenie generatora
			maska = 0x00 ;
			PORTD &= 0xFC ;
		}
		
		//wylaczanie przetwornicy
		przetwornica++ ;
		if( (bit_is_clear(PINC, 0) && bit_is_clear(PINC, 1)) || (przetwornica > 30000) ){
			cbi(PORTD, 4) ;
		}
		
		//monitorowanie stanu baterii
		if(bit_is_clear(PIND, 3)){
			//komunikat
			wysw(LCD_BAT) ;
			for(unsigned char i=0 ; i<100 ; i++){
				asm("wdr") ;
				_delay_ms(10) ;
			}
			
			//wylaczenie
			cbi(PORTD, 4) ;
		}
		
		//opoznienie
		_delay_ms(1) ;
	}
}
