/********************************************************************************************
(C) Jakub Borzdynski 2006
jotbeage@interia.pl
-----------------------------
Szkola Konstruktorow 137 - g-meter
-----------------------------
Wykonywanie pomiarow za pomoca czujnika przyspieszenia.
********************************************************************************************/
#ifndef jb_adxl_cpp
#define jb_adxl_cpp
#include "adxl.h"

//-------------------------------------------------------------------------------------------------------------
adxl::adxl(){
	//przygotuj ADC
	ADMUX = 0x00 ;
	ADCSRA = 0x83 ;
	
	//zerowanie zmiennych
	ile  = 15; //standardowo
	zliczono = 0 ; 
	
	//wartosci stale - odczyt nastaw z EEPROM
	x[0] = pamiec[EEPROM_ADR_X_MIN];
	x[1] = pamiec[EEPROM_ADR_X_MAX];
	x[2] = pamiec[EEPROM_ADR_X_MIN_AMP];
	x[3] = pamiec[EEPROM_ADR_X_MAX_AMP];

	y[0] = pamiec[EEPROM_ADR_Y_MIN];
	y[1] = pamiec[EEPROM_ADR_Y_MAX];
	y[2] = pamiec[EEPROM_ADR_Y_MIN_AMP];
	y[3] = pamiec[EEPROM_ADR_Y_MAX_AMP];
}

//-------------------------------------------------------------------------------------------------------------
unsigned int adxl::zmierz(unsigned char nr_kanalu){
	//ustaw kanal
	ADMUX &= 0xF0 ;
	ADMUX |= (nr_kanalu&0x0F) ;
	
	//rozpocznij konwersje
	sbi(ADCSRA, ADSC) ;
	while(bit_is_set(ADCSRA, ADSC)){;}
	//sbi(SREG, SREG_I) ;
	
	//zwroc rezultat
	return ADCL + ADCH*256 ;
}

//-------------------------------------------------------------------------------------------------------------
unsigned int adxl::pomiar_usredniony(unsigned char nr_kanalu){
	//ustawienie zmiennych
	suma[0] = 0 ;
	zliczono =  0 ;
	ile = 50 ;
	
	while(1){
		suma[0] += zmierz(nr_kanalu) ;
		asm("wdr") ;
		_delay_ms(10) ;
		
		if(++zliczono==ile){
			srednia[0] = suma[0] / zliczono ;
			suma[0] = 0 ;
			zliczono =  0 ;
			return srednia[0];
		}
	}
}

//-------------------------------------------------------------------------------------------------------------
void adxl::konfiguracja(){
	//ustawienie poziomu odniesienia na osi X
	wysw.pozycja(0) ;
	while(bit_is_set(PINC, 1)){
		//oczekiwanie
		wysw(pomiar_usredniony(KANAL_X)) ;
	}
	
	//ustawienie poziomu odniesienia na osi Y
	wysw.pozycja(1) ;
	while(bit_is_set(PINC, 1)){
		//oczekiwanie
		wysw(pomiar_usredniony(KANAL_Y)) ;
	}
	
	//regulacja parametru Xmin
	wysw.pozycja(2) ;
	while(bit_is_set(PINC, 1)){
		//oczekiwanie
		wysw(pomiar_usredniony(KANAL_X)) ;
	}
	
	//zapamietaj
	x[0] = pomiar_usredniony(KANAL_X) ;
	x[2] = pomiar_usredniony(KANAL_X_AMP) ;
	
	//regulacja parametru Xmax
	wysw.pozycja(3) ;
	while(bit_is_set(PINC, 1)){
		//oczekiwanie
		wysw(pomiar_usredniony(KANAL_X)) ;
	}
	
	//zapamietaj
	x[1] = pomiar_usredniony(KANAL_X) ;
	x[3] = pomiar_usredniony(KANAL_X_AMP) ;
	
	
	//regulacja parametru Ymin
	wysw.pozycja(4) ;
	while(bit_is_set(PINC, 1)){
		//oczekiwanie
		wysw(pomiar_usredniony(KANAL_Y)) ;
	}
	
	//zapamietaj
	y[0] = pomiar_usredniony(KANAL_Y) ;
	y[2] = pomiar_usredniony(KANAL_Y_AMP) ;
	
	//regulacja parametru Ymax
	wysw.pozycja(5) ;
	while(bit_is_set(PINC, 1)){
		//oczekiwanie
		wysw(pomiar_usredniony(KANAL_Y)) ;
	}
	
	//zapamietaj
	y[1] = pomiar_usredniony(KANAL_Y) ;
	y[3] = pomiar_usredniony(KANAL_Y_AMP) ;
	
	//koniec konfiguracji i zapis w EEPROM
	wysw.pozycja(6) ;
	pamiec.zapisz(EEPROM_ADR_X_MIN, 	x[0]);
	pamiec.zapisz(EEPROM_ADR_X_MAX,		x[1]);
	pamiec.zapisz(EEPROM_ADR_X_MIN_AMP, x[2]);
	pamiec.zapisz(EEPROM_ADR_X_MAX_AMP,	x[3]);

	pamiec.zapisz(EEPROM_ADR_Y_MIN, 	y[0]);
	pamiec.zapisz(EEPROM_ADR_Y_MAX,		y[1]);
	pamiec.zapisz(EEPROM_ADR_Y_MIN_AMP,	y[2]);
	pamiec.zapisz(EEPROM_ADR_Y_MAX_AMP, y[3]);
}

//-------------------------------------------------------------------------------------------------------------
unsigned int adxl::operator()(char tryb){
	//pomiary
	suma[0] += zmierz(KANAL_X) ; //pomiar na kanale X
	suma[1] += zmierz(KANAL_Y) ; //pomiar na kanale Y
	suma[2] += zmierz(KANAL_X_AMP) ; //pomiar na kanale Xamp
	suma[3] += zmierz(KANAL_Y_AMP) ; //pomiar na kanale Yamp
	
	//obliczenia
	if(++zliczono==ile){
		//wyznaczenie pomiaru
		for(unsigned char t=0 ; t<4 ; t++){
			srednia[t] = suma[t] / zliczono ;
			suma[t] = 0 ;
		}
		zliczono =  0 ;
		
		//-------------------------------------------------------------------
		//POMIAR ZALEZNY OD TRYBU
		
		switch(tryb){
		
			//-------------------------------------------------------------------
			//POMIAR_G_MAX - maksymalne przyspieszenie
			case POMIAR_G_MAX:
				//zerowac stary pomiar ?
				if(bit_is_clear(PINC, 1)){ostatni_pomiar = 0 ;}
				
				//zmien liczbe usrednianych pomiarow
				ile = 20 ;
				
				//oblicz modul
				srednia[1] = (y[1]+y[0])/2 - srednia[1] ;
				if(srednia[1]&0x8000){srednia[1]=~srednia[1]+1 ;}
				
				//przeliczenie na g
				srednia[1] = srednia[1]*100 / ((y[1]-y[0])/2) ;
				
				//porownanie
				if((ostatni_pomiar&0x7FFF) < srednia[1]) { ostatni_pomiar = srednia[1] ; }
				
				//wymus wyswietlenie kropki na wyswietlaczu
				ostatni_pomiar |= 0x8000 ;
			break ;
			
			//-------------------------------------------------------------------
			//POMIAR_G - przyspieszenie chwilowe
			case POMIAR_G:
				//zmien domyslne zliczanie
				ile = 50 ;
				
				//oblicz modul
				srednia[1] = (y[1]+y[0])/2 - srednia[1] ;
				if(srednia[1]&0x8000){srednia[1]=~srednia[1]+1 ;}
				
				//przeliczenie na g
				srednia[1] = srednia[1]*100 / ((y[1]-y[0])/2) ;
				
				//wymus wyswietlenie kropki na wyswietlaczu
				ostatni_pomiar = srednia[1] | 0x8000 ;
			break ;
			
			//-------------------------------------------------------------------
			//POMIAR_ST - wyznaczanie wychylenia katowego przyrzadu
			case POMIAR_ST:
				//zmien domyslne zliczanie
				ile = 100 ;
				
				//zmienne
				double a, b, w ;
				
				//oblicz modul
				a = (x[3]+x[2])/2.0 - srednia[2] ;
				b = (y[3]+y[2])/2.0 - srednia[3] ;
				
				//obliczenia
				w = 180 + atan2(a, b)*57.29 ;
				ostatni_pomiar = w ;
				ostatni_pomiar %= 360 ;
			break ;
		}
	}
	
	//zwroc rezultat
	return ostatni_pomiar ;
}


#endif