/********************************************************************************************
(C) Jakub Borzdynski 2006
jotbeage@interia.pl
-----------------------------
Szkola Konstruktorow 137 - g-meter
-----------------------------
Obsluga wyswietlacza
********************************************************************************************/
#ifndef jb_lcd_cpp
#define jb_lcd_cpp
#include "lcd.h"


unsigned char ttt = 0 ;												 //10   11   12   13   14   15
//tablice konwersji dla portow (dwa rodzaje polaczen)                //b  //t  //C  //n  //P  //c
char tab_B1[20] = {71 , 247, 143, 167, 55 , 39 , 7  , 247, 7,   39 , 7  , 15 , 79 , 151, 31 , 143};
char tab_D1[20] = {159, 191, 159, 159, 191, 223, 223, 159, 159, 159, 255, 255, 223, 255, 159, 255};
																	 //A  //f  //r
char tab_B2[20] = {35 , 175, 51 , 39 , 175, 103, 99 , 47 , 35 , 39 , 43 , 123, 251};
char tab_D2[20] = {223, 255, 191, 191, 159, 159, 159, 255, 159, 159, 159, 159, 191};

//-------------------------------------------------------------------------------------------------------------
//konstruktor
lcd::lcd(){
	//ustaw porty
	PORTD |= 0xE0 ;
	DDRD  |= 0xE0 ;
	PORTB |= 0xFF ;
	DDRB  |= 0xFF ;
	
	//konfiguracja licznika T0
	TCCR0B = 0x02 ;
	sbi(TIMSK0, TOIE0) ;
	sbi(SREG, SREG_I) ;
}


//zapisuje wartosc
void lcd::operator()(unsigned int wartosc){
	//okreslenie, czy potrzebna jest kropka
	kropka = wartosc&0x8000 ;
	wartosc &= 0x7FFF ;
	
	//sprawdz znaki specjalne
	if(wartosc==LCD_BAT){
		ekran[2] = 10 ; //b
		ekran[1] = 10 ; //A
		ekran[0] = 11 ; //t
	}
	if(wartosc==LCD_CFN){
		ekran[2] = 12 ; //N
		ekran[1] = 11 ; //F
		ekran[0] = 13 ; //C
	}
	if(wartosc==LCD_PRC){
		ekran[2] = 14 ; //P
		ekran[1] = 12 ; //r
		ekran[0] = 15 ; //c
	}
	
	//sprawdz przekroczenie zakresu
	if(wartosc>999){return;}
	
	//konwersja
	for(unsigned char i=0 ; i<3 ; i++){
		ekran[i] = wartosc%10 ; wartosc /= 10 ;
	}
}


//odswieza "ekran"
void lcd::refresh(){
	//wylacz wszystkie wyswietlacze i wygas segmenty
	PORTD |= 0xE0 ;
	PORTB |= 0xFF ;
	
	//wysterowanie wyswietlacza
	switch(++wsk%3){
		case 0: //WYS 1
			//wlacz odpowiedni tranzystor
			cbi(PORTD, 7) ;
			
			//wlacz segmenty
			PORTB &= tab_B1[ekran[0]] ;
			PORTD &= tab_D1[ekran[0]] ;
		
			break ;
			
		case 1: //WYS 2
			//wlacz odpowiedni tranzystor
			cbi(PORTB, 1) ;
			
			//wlacz segmenty
			PORTB &= tab_B2[ekran[1]] ;
			PORTD &= tab_D2[ekran[1]] ;			
			break ;
			
		case 2: //WYS 3
			//wlacz odpowiedni tranzystor
			cbi(PORTB, 0) ;
			
			//wlacz segmenty
			PORTB &= tab_B1[ekran[2]] ;
			PORTD &= tab_D1[ekran[2]] ;			
			
			//wyswietlac kropke ?
			PORTB &= ~(0x04*kropka) ;
			
			break ;
	}
}


//wylacz wyswietlacze
void lcd::off(){
	PORTD |= 0xE0 ;
	PORTB |= 0xFF ;
}

//pokazuje etapy konfiguracji
void lcd::pozycja(unsigned char poz){
	//wpisz numer programu
	ekran[2] = 14 ; //P
	ekran[1] = 0 ; //0
	ekran[0] = poz%10 ; //poz
	
	//zwloka czasowa
	for(unsigned char i=0 ; i<75 ; i++){
		asm("wdr") ;
		_delay_ms(10) ;
	}
}

//utworz obiekt
lcd wysw ;

#endif